transcript.to.utr.range = function( ids, end=c( 'both', '5', '3' ) ) {
	ids = .get.correct.column( 'transcript', ids )
	if( is.null( ids ) ) {
		return( NULL )
	}
	transcripts = as.data.frame( transcript.details( ids ) )
	if( dim( transcripts )[1] == 0 ) {
		return( NULL )
	}
	end = match.arg( end )
	
	exons = transcript.to.exon( ids, as.vector='data.frame' )
	exons[,'sequence'] = NULL
	
	all.data   = merge( exons, transcripts, by.x='IN1', by.y='stable_id' )
	all.data.3 = all.data[ !is.na( all.data[,"translation_end_exon"] )   & ( all.data[,"translation_end_exon"]   == all.data[,"exon_id"] ), ]
	all.data.5 = all.data[ !is.na( all.data[,"translation_start_exon"] ) & ( all.data[,"translation_start_exon"] == all.data[,"exon_id"] ), ]
	
	fwds.5 = apply( all.data.5[ all.data.5$"strand.y" == 1, c( "start.y", "start.x", "translation_start", "chromosome_name", "IN1" ) ], 1,
				   function(a) { list( IN1=as.character(a[5]), space=as.character(a[4]), start=as.numeric(a[1]), end=as.numeric(a[2]) + as.numeric(a[3]), strand=1, prime='5' ) } )
	fwds.3 = apply( all.data.3[ all.data.3$"strand.y" == 1,c( "end.y", "start.x", "translation_end", "chromosome_name", "IN1" ) ], 1,
				   function(a) { list( IN1=as.character(a[5]), space=as.character(a[4]), start=as.numeric(a[2]) + as.numeric(a[3]), end=as.numeric(a[1]), strand=1, prime='3' ) } )
	revs.5 = apply( all.data.5[ all.data.5$"strand.y" == -1, c( "end.y", "end.x", "translation_start", "chromosome_name", "IN1" ) ], 1,
				   function(a) { list( IN1=as.character(a[5]), space=as.character(a[4]), start=as.numeric(a[2]) - as.numeric(a[3]), end=as.numeric(a[1]), strand=-1, prime='5' ) } )
	revs.3 = apply( all.data.3[ all.data.3$"strand.y" == -1,c( "start.y", "end.x", "translation_end", "chromosome_name", "IN1" ) ], 1,
				   function(a) { list( IN1=as.character(a[5]), space=as.character(a[4]), start=as.numeric(a[1]), end=as.numeric(a[2]) - as.numeric(a[3]), strand=-1, prime='3' ) } )
	
	if( end == '5' ) {
		df = as.data.frame( do.call( rbind, lapply( c( fwds.5, revs.5 ), unlist ) ) )
	}	else if( end == '3' ) {
		df = as.data.frame( do.call( rbind, lapply( c( fwds.3, revs.3 ), unlist ) ) )
	}	else { # both
		df = as.data.frame( do.call( rbind, lapply( c( fwds.5, fwds.3, revs.5, revs.3 ), unlist ) ) )
	}
	df$IN1    = as.character( df$IN1 )
	df$start  = as.numeric( as.character( df$start ) )
	df$end    = as.numeric( as.character( df$end ) )
	df$strand = as.numeric( as.character( df$strand ) )
	as( df, 'RangedData' )
}

transcript.to.coding.range = function( ids, end=c( 'both', '5', '3' ) ) {
	ids = .get.correct.column( 'transcript', ids )
	if( is.null( ids ) ) {
		return( NULL )
	}
	utrs = transcript.to.utr.range( ids, end )
	if( is.null( utrs ) ) {
		return( NULL )
	}
	transcripts = transcript.details( ids )
	process.fn  = function( stable_id, start, end, space, strand, biotype, status, description, db_display_name, symbol,
						   symbol_description, translation_start, translation_start_exon, translation_end, translation_end_exon ) {
		specutr = utrs[ as.character(utrs$'IN1') == stable_id, ]
		if( length( specutr ) > 0 ) {
			prime.3 = specutr[specutr$'prime' == '3',]
			prime.5 = specutr[specutr$'prime' == '5',]
			if( strand == 1 ) {
				if( dim( prime.3 )[1] > 0 ) {
					end = end - width( prime.3 )
				}
				if( dim( prime.5 )[1] > 0 ) {
					start = start + width( prime.5 )
				}
			}
			else {
				if( dim( prime.3 )[1] > 0 ) {
					start = start + width( prime.3 )
				}
				if( dim( prime.5 )[1] > 0 ) {
					end = end - width( prime.5 )
				}
			}
		}
		RangedData( ranges=IRanges( start=start, end=end ), stable_id=stable_id, space=space, strand=strand, biotype=biotype, status=status,
				   description=description, db_display_name=db_display_name, symbol=symbol, symbol_description=symbol_description,
				   translation_start=translation_start, translation_start_exon=translation_start_exon, translation_end=translation_end,
				   translation_end_exon=translation_end_exon )
	}
	filter.list = c( stable_id="stable_id", start="start", end="end", space="space", strand="strand", biotype="biotype", status="status",
					description="description", db_display_name="db_display_name", symbol="symbol", symbol_description="symbol_description",
					translation_start="translation_start", translation_start_exon="translation_start_exon", translation_end="translation_end",
					translation_end_exon="translation_end_exon" ) 
	coerce.list = c( as.character, as.integer, as.integer, as.character, as.integer, as.character, as.character,
					as.character, as.character, as.character, as.character,
					as.integer, as.integer, as.integer,
					as.integer )
	xmap.range.apply( transcripts, process.fn, filter=filter.list, coerce=coerce.list )
}

.probeset.filtering = function( probesets, transcripts, end=c( 'both', '5', '3' ), fn ) {
    if( missing( probesets ) ) probesets = NULL
	if( missing( transcripts ) ) transcripts = NULL
	end = match.arg( end )
	
	probesets = .get.correct.column( 'probeset', probesets )
	transcripts = .get.correct.column( 'transcript', transcripts )
	
	if( is.null( probesets ) && is.null( transcripts ) ) {
		return( NULL )
	}
	else if( is.null( probesets ) ) {
		probesets = transcript.to.probeset( transcripts )
		if( is.null( probesets ) ) {
			return( NULL )
		}
	}
	else if( is.null( transcripts ) ) {
		transcripts = probeset.to.transcript( probesets, as.vector='data.frame' )
		keep = !( is.na( transcripts$"translation_start" ) )
		transcripts = unique( transcripts[keep,]$'stable_id' )
		if( length( transcripts ) == 0 ) {
			return( NULL )
		}
	}
	else {
		tpts = probeset.to.transcript( probesets, as.vector='data.frame' )
		keep = !( is.na( tpts$"translation_start" ) )
		tpts = tpts[keep,]
		keep = tpts$"stable_id" %in% transcripts
		tpts = tpts[ keep, ]
		if( dim( tpts )[1] == 0 ) {
			return( NULL )
		}
		transcripts = unique( tpts$'stable_id' )
	}
	probesets = unreliable( probesets, exclude=TRUE )
	ranges = fn( transcripts, end=end )
	psts = unique( xmap.range.apply( ranges, probesets.in.range ) )
	keep = psts %in% probesets
	psts = psts[ keep ]
	if( length( psts ) == 0 ) {
		NULL
	}
	else {
		psts
	}
}

utr.probesets = function( probesets, transcripts, end=c( 'both', '5', '3' ) ) {
	.probeset.filtering( probesets, transcripts, end, transcript.to.utr.range )
}

coding.probesets = function( probesets, transcripts, end=c( 'both', '5', '3' ) ) {
	.probeset.filtering( probesets, transcripts, end, transcript.to.coding.range )
}
