# Testing all of the statements (to make sure none of them crash)
# Testing for acutal responses is more tricky, as the responses may change from one
# release to the next...

if(FALSE) {
  library( "RUnit" )
  library( "xmapcore" )
}

###############################################################################
## Utility methods
##
.get.functions = function( func.type, func.pattern, min.len, name.build.function ) {
	.functions = ls( envir=xmapcore:::.xmap.queries )
	.queries = strsplit( .functions[ grep( func.pattern, .functions ) ], '\\.' )
	checkTrue( length( .queries ) >= min.len, paste( "Should have at least ", min.len, " '", func.type, "' functions", sep="" ) )
	r = lapply( .queries, function(i) {
		name = name.build.function( i )
		c( name, getFunction( name ) )
	} )
	r
}

.run.full.test.suite = function() {
	1 == length( grep( 'picr.man.ac.uk', Sys.info()[ 'nodename' ] ) )
}

.run.functions = function( functions, executor ) {
	lapply( c( 'hs-test', 'mm-test', 'rn-test', 'pb-test' ), function( db ) {
		data = tryCatch( xmap.connect( db ), error=function(e) { FALSE } )
		if( length( data ) == 1 ) {
			print( paste( "Cannot find datasource '", db, "', so skipping these tests.", sep="" ) )
		}
		else {
			# Set this to true if this species has no arrays
			no.arrays = is.null( all.arrays( as.vector=F ) )
			lapply( functions, function( func ) {
				if( no.arrays && length( grep( "array|probe|hit|est_|prediction", func[[1]] ) ) == 1 ) {
					print( paste( "Skipping", func[[1]], "on", xmapcore:::.xmap.internals$db.name ) )
				}
				else {
					print( paste( "Running", func[[1]], "on", xmapcore:::.xmap.internals$db.name ) )
					tryCatch( executor( func[[1]], func[[2]] ), error=function(e) {
						traceback()
						print( e )
					} )
				}
			} )
		}
	} )
}

###############################################################################
## SetUp is called before each test method
##
.setUp = function() {
}

###############################################################################
## TearDown is called after each test method
##
.tearDown = function() {
	xmap.disconnect()
}

test.gotsomefunctions = function() {
  checkTrue( length( ls( envir=xmapcore:::.xmap.queries ) ) >= 96, "Should have at least 96 functions" )
}

test.to.queries = function() {
	# Get all 'to' functions (of which there should be at least 58)
	.functions = .get.functions( 'to', '\\.to\\.', 58, function(i) { paste( i[1], '.to.', i[3], sep="" ) } )

	# And run them, searching for the thing called 'InvalidId' which should return no results, but not crash)
    .run.functions( .functions,  function( name, f ) {
		if( !.run.full.test.suite() && name == "array.to.probeset" ) {
			print( '\nSkipping array.to.probeset test, as not on a PICR machine' )
		}
		else {
			r = f( 'InvalidId' )
			if( name == "array.to.probeset" ) {
				checkTrue( length( r ) > 0, "should have some probesets for an array" )
			}
			else {
				checkTrue( is.null( r ), "should return null" )
			}
		}
	} )
}